@@ -227,7 +227,7 @@ static char *configlet_get_configured_name(ConfigletList *cl)
 
 	g_return_val_if_fail(optkey, NULL);
 	return options_lookup_string_with_default(cl->cg->options, optkey,
-			"Default");
+			strcmp(cl->family, "Colours") ? "Default" : "GTK");
 }
 
 static void configlet_list_build(ConfigletList *cl)
@@ -639,13 +639,15 @@ static void configlet_rename(ConfigletList *cl,
 
 static void edit_thing_by_name(ConfigletList *cl, const char *name)
 {
+    GdkScreen *scrn = gtk_widget_get_screen(cl->cg->widget);
+    
 	if (!strcmp(cl->family, "Profiles"))
 	{
-		profilegui_open(name);
+		profilegui_open(name, scrn);
 	}
 	else if (!strcmp(cl->family, "Colours"))
 	{
-		colourgui_open(name);
+		colourgui_open(name, scrn);
 	}
 	else if (!strcmp(cl->family, "Shortcuts"))
 	{
@@ -929,11 +931,13 @@ static void configlet_setup_family(ConfigletData *cg, ConfigletList *cl,
 	g_free(wbasename);
 }
 
-gboolean configlet_open(void)
+gboolean configlet_open(GdkScreen *scrn)
 {
 	
 	if (configlet_data)
 	{
+	    if (scrn)
+            gtk_window_set_screen(GTK_WINDOW(configlet_data->widget), scrn);
 		gtk_window_present(GTK_WINDOW(configlet_data->widget));
 	}
 	else
@@ -949,6 +953,8 @@ gboolean configlet_open(void)
 			configlet_delete(cg);
 			return FALSE;
 		}
+	    if (scrn)
+            gtk_window_set_screen(GTK_WINDOW(cg->widget), scrn);
 		configlet_glade = cg->glade;
 
 		cg->options = options_open("Global", "roxterm options");
