@@ -76,7 +76,8 @@ static gboolean global_options_show_usage(const gchar *option_name,
 	  "    [--profile=PROFILE|-p PROFILE] \n"
 	  "    [--colour-scheme=SCHEME|--color-scheme=SCHEME|-c SCHEME]\n"
 	  "    [--shortcut-scheme=SCHEME|-s SCHEME]\n"
-	  "    [--fullscreen|-f] [--zoom=ZOOM|-z ZOOM] [--title=TITLE|-T TITLE]\n"
+	  "    [--fullscreen|-f] [--zoom=ZOOM|-z ZOOM]\n"
+	  "    [--title=TITLE|-T TITLE] [--tab-name=NAME|-n NAME]\n"
 	  "    [--separate] [--replace] [--tab]\n"
 	  "    [--directory=DIRECTORY|-d DIRECTORY]\n"
 	  "    [--show-menubar] [--hide-menubar] [-e|--execute COMMAND]\n");
@@ -117,10 +118,10 @@ static gboolean global_options_set_directory(const gchar *option_name,
 	}
 	else
 	{
-		char *cwd = g_get_current_dir();
+		char *cwd = GET_CURRENT_DIR();
 
 		global_options_directory = g_build_filename(cwd, value, NULL);
-		g_free(cwd);
+		FREE_CURRENT_DIR(cwd);
 	}
 	if (!g_file_test(global_options_directory, G_FILE_TEST_IS_DIR))
 	{
@@ -217,6 +218,9 @@ static GOptionEntry global_g_options[] = {
 	{ "title", 'T', G_OPTION_FLAG_IN_MAIN,
 		G_OPTION_ARG_CALLBACK, global_options_set_string,
 		N_("Set window title"), N_("TITLE") },
+	{ "tab-name", 'n', G_OPTION_FLAG_IN_MAIN,
+		G_OPTION_ARG_CALLBACK, global_options_set_string,
+		N_("Set tab name"), N_("NAME") },
 	{ "tab", 0, G_OPTION_FLAG_IN_MAIN,
 		G_OPTION_ARG_NONE, &global_options_tab,
 		N_("Open a tab in an existing window instead of\n"
@@ -371,7 +375,11 @@ void global_options_init(int *argc, char ***argv, gboolean report)
 			}
 			else if (g_file_test("roxterm-config", G_FILE_TEST_IS_EXECUTABLE))
 			{
-				global_options_bindir = g_get_current_dir();
+			    /* Paranoia in case g_free() != free() */
+			    char *bindir = GET_CURRENT_DIR();
+			    
+				global_options_bindir = g_strdup(bindir);
+				FREE_CURRENT_DIR(bindir);
 			}
 			else if (!g_find_program_in_path("roxterm-config"))
 			{
@@ -380,11 +388,11 @@ void global_options_init(int *argc, char ***argv, gboolean report)
 		}
 		else if (!g_path_is_absolute(global_options_bindir))
 		{
-			/* Partical path, must be relative to current dir */
-			char *cur = g_get_current_dir();
+			/* Partial path, must be relative to current dir */
+			char *cur = GET_CURRENT_DIR();
 			char *full = g_build_filename(cur, global_options_bindir, NULL);
 
-			g_free(cur);
+			FREE_CURRENT_DIR(cur);
 			g_free(global_options_bindir);
 			global_options_bindir = full;
 		}
