@@ -147,20 +147,43 @@ static void menutree_build_shell(MenuTree *menu_tree, GtkMenuShell * shell, ...)
 	va_end(ap);
 }
 
+#define URI_MENU_ITEMS \
+        _("_Open URL in browser"), MENUTREE_OPEN_IN_BROWSER, \
+        _("_Send email"), MENUTREE_OPEN_IN_MAILER, \
+        _("_Copy address to clipboard"), MENUTREE_COPY_URI, \
+        _("_"), MENUTREE_URI_SEPARATOR
+        
 #define TOP_LEVEL_MENU_ITEMS \
-                _("_File"), MENUTREE_FILE, \
-                _("_Edit"), MENUTREE_EDIT, \
-                _("_View"), MENUTREE_VIEW, \
-                _("_Preferences"), MENUTREE_PREFERENCES, \
-                _("Ta_bs"), MENUTREE_TABS, \
-                _("_Help"), MENUTREE_HELP, \
-                NULL
+        _("_File"), MENUTREE_FILE, \
+        _("_Edit"), MENUTREE_EDIT, \
+        _("_View"), MENUTREE_VIEW, \
+        _("_Preferences"), MENUTREE_PREFERENCES, \
+        _("Ta_bs"), MENUTREE_TABS, \
+        _("_Help"), MENUTREE_HELP
 
+#define COPY_PASTE_MENU_ITEMS \
+		_("_Copy"), MENUTREE_EDIT_COPY, \
+		_("_Paste"), MENUTREE_EDIT_PASTE, \
+		_("Cop_y & Paste"), MENUTREE_EDIT_COPY_AND_PASTE, \
+		"_", MENUTREE_NULL_ID
+
+#define RESET_MENU_ITEMS \
+		_("_Reset"), MENUTREE_EDIT_RESET, \
+		_("Reset And C_lear"), MENUTREE_EDIT_RESET_AND_CLEAR
+		
+#define SHOW_MENU_BAR_ITEM \
+		_("Show Menu_bar"), MENUTREE_VIEW_SHOW_MENUBAR
+		
+#define ENC_INPUT_ITEMS \
+        _("C_haracter Encoding"), MENUTREE_PREFERENCES_CHARACTER_ENCODING, \
+		_("_Input Methods"), MENUTREE_PREFERENCES_INPUT_METHODS
+		
 GtkMenu *menutree_submenu_from_id(MenuTree *mtree, MenuTreeID id)
 {
 	GtkWidget *item = menutree_get_widget_for_id(mtree, id);
 
-	return GTK_MENU(gtk_menu_item_get_submenu(GTK_MENU_ITEM(item)));
+	return item ?
+	        GTK_MENU(gtk_menu_item_get_submenu(GTK_MENU_ITEM(item))) : NULL;
 }
 
 static char *get_accel_path(Options *shortcuts, const char *branch_name)
@@ -173,9 +196,12 @@ static void
 menutree_set_accel_path_for_submenu(MenuTree *mtree, MenuTreeID id,
 		const char *menu_branch)
 {
-	char *accel_path = get_accel_path(mtree->shortcuts, menu_branch);
 	GtkMenu *menu = menutree_submenu_from_id(mtree, id);
-
+	char *accel_path;
+	
+	if (!menu)
+	    return;
+	accel_path = get_accel_path(mtree->shortcuts, menu_branch);
 	gtk_menu_set_accel_path(menu, accel_path);
 	g_free(accel_path);
 	gtk_menu_set_accel_group(menu, mtree->accel_group);
@@ -185,10 +211,13 @@ static void
 menutree_set_accel_path_for_item(MenuTree * tree, MenuTreeID id,
 	const char *path_leaf)
 {
-	char *full_path = get_accel_path(tree->shortcuts, path_leaf);
-
-	gtk_menu_item_set_accel_path(GTK_MENU_ITEM(tree->item_widgets[id]),
-		full_path);
+    GtkWidget *item = tree->item_widgets[id];
+	char *full_path;
+	
+	if (!item)
+	    return;
+	full_path = get_accel_path(tree->shortcuts, path_leaf);
+	gtk_menu_item_set_accel_path(GTK_MENU_ITEM(item), full_path);
 	g_free(full_path);
 }
 
@@ -198,7 +227,7 @@ menutree_set_accel_path_for_tab(MenuTree * tree, int tab)
 	char *leaf = NULL;
 	char *full_path = NULL;
 
-	if (tab >= 10)
+	if (!tree->tabs || tab >= 10)
 		return;
 	if (!tree->disable_tab_shortcuts && tree->ntabs != 1)
 	{
@@ -224,6 +253,8 @@ static void menutree_apply_tab_shortcuts(MenuTree *mtree)
 
 void menutree_apply_shortcuts(MenuTree *tree, Options *shortcuts)
 {
+    GtkMenu *submenu;
+    
 	tree->shortcuts = shortcuts;
 	shortcuts_enable_signal_handler(FALSE);
 	menutree_set_accel_path_for_submenu(tree, MENUTREE_FILE, "File");
@@ -233,8 +264,9 @@ void menutree_apply_shortcuts(MenuTree *tree, Options *shortcuts)
 			"Preferences");
 	/* Tabs have accelerators set dynamically so set paths
 	 * for Next and Previous Tab individually */
-	gtk_menu_set_accel_group(menutree_submenu_from_id(tree, MENUTREE_TABS),
-			tree->accel_group);
+    submenu = menutree_submenu_from_id(tree, MENUTREE_TABS);
+    if (submenu)
+        gtk_menu_set_accel_group(submenu, tree->accel_group);
 	menutree_set_accel_path_for_item(tree, MENUTREE_TABS_NEXT_TAB,
 			"Tabs/Next Tab");
 	menutree_set_accel_path_for_item(tree, MENUTREE_TABS_PREVIOUS_TAB,
@@ -424,15 +456,12 @@ static void menutree_build(MenuTree *menu_tree, Options *shortcuts,
 	if (menu_type == GTK_TYPE_MENU_BAR)
 	{
 		menutree_build_shell(menu_tree, GTK_MENU_SHELL(menu_tree->top_level),
-				TOP_LEVEL_MENU_ITEMS);
+				TOP_LEVEL_MENU_ITEMS, NULL);
 	}
 	else
 	{
 		menutree_build_shell(menu_tree, GTK_MENU_SHELL(menu_tree->top_level),
-			_("_Open URL in browser"), MENUTREE_OPEN_IN_BROWSER,
-			_("_Send email"), MENUTREE_OPEN_IN_MAILER,
-			_("_Copy address to clipboard"), MENUTREE_COPY_URI,
-			_("_"), MENUTREE_URI_SEPARATOR, TOP_LEVEL_MENU_ITEMS);
+		    URI_MENU_ITEMS, TOP_LEVEL_MENU_ITEMS, NULL);
 	}
 
 	submenu = gtk_menu_new();
@@ -446,18 +475,15 @@ static void menutree_build(MenuTree *menu_tree, Options *shortcuts,
 
 	submenu = gtk_menu_new();
 	menutree_build_shell(menu_tree, GTK_MENU_SHELL(submenu),
-		_("_Copy"), MENUTREE_EDIT_COPY,
-		_("_Paste"), MENUTREE_EDIT_PASTE,
-		"_", MENUTREE_NULL_ID,
+	    COPY_PASTE_MENU_ITEMS,
 		_("_Set Window Title..."), MENUTREE_EDIT_SET_WINDOW_TITLE,
-		_("_Reset"), MENUTREE_EDIT_RESET,
-		_("Reset And C_lear"), MENUTREE_EDIT_RESET_AND_CLEAR, NULL);
+		RESET_MENU_ITEMS, NULL);
 	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_tree->item_widgets
 			[MENUTREE_EDIT]), submenu);
 
 	submenu = gtk_menu_new();
 	menutree_build_shell(menu_tree, GTK_MENU_SHELL(submenu),
-		_("Show Menu_bar"), MENUTREE_VIEW_SHOW_MENUBAR,
+        SHOW_MENU_BAR_ITEM,
 		_("_Full Screen"), MENUTREE_VIEW_FULLSCREEN,
 		"_", MENUTREE_NULL_ID,
 		_("Zoom _In"), MENUTREE_VIEW_ZOOM_IN,
@@ -479,23 +505,24 @@ static void menutree_build(MenuTree *menu_tree, Options *shortcuts,
 		_("_Edit Current Profile"), MENUTREE_PREFERENCES_EDIT_CURRENT_PROFILE,
 		_("E_dit Current Colour Scheme"),
 			MENUTREE_PREFERENCES_EDIT_CURRENT_COLOUR_SCHEME,
-		/*_("Edi_t Current Shortcuts Scheme"),
-			MENUTREE_PREFERENCES_EDIT_CURRENT_SHORTCUTS,*/
 		"_", MENUTREE_NULL_ID,
 		_("Configuration _Manager"), MENUTREE_PREFERENCES_CONFIG_MANAGER,
 		"_", MENUTREE_NULL_ID,
-		   _("C_haracter Encoding"),
-		   MENUTREE_PREFERENCES_CHARACTER_ENCODING,
-		_("_Input Methods"), MENUTREE_PREFERENCES_INPUT_METHODS, NULL);
+		ENC_INPUT_ITEMS,
+		NULL);
 	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_tree->item_widgets
 			[MENUTREE_PREFERENCES]), submenu);
 
 	submenu = gtk_menu_new();
 	menutree_build_shell(menu_tree, GTK_MENU_SHELL(submenu),
 		_("Na_me Tab..."), MENUTREE_TABS_NAME_TAB,
-		"_", MENUTREE_TABS_SEP1,
+		"_", MENUTREE_NULL_ID,
 		_("_Previous Tab"), MENUTREE_TABS_PREVIOUS_TAB,
-		_("_Next Tab"), MENUTREE_TABS_NEXT_TAB, "_", MENUTREE_NULL_ID, NULL);
+		_("_Next Tab"), MENUTREE_TABS_NEXT_TAB,
+		"_", MENUTREE_NULL_ID,
+		_("Move Tab _Left"), MENUTREE_TABS_MOVE_TAB_LEFT,
+		_("Move Tab _Right"), MENUTREE_TABS_MOVE_TAB_RIGHT,
+		"_", MENUTREE_NULL_ID, NULL);
 	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_tree->item_widgets
 			[MENUTREE_TABS]), submenu);
 
@@ -508,6 +535,20 @@ static void menutree_build(MenuTree *menu_tree, Options *shortcuts,
 	menutree_apply_shortcuts(menu_tree, shortcuts);
 }
 
+static void menutree_build_short_popup(MenuTree *menu_tree, Options *shortcuts,
+		GType menu_type)
+{
+	menu_tree->top_level = gtk_menu_new();
+    menutree_build_shell(menu_tree, GTK_MENU_SHELL(menu_tree->top_level),
+        URI_MENU_ITEMS,
+        COPY_PASTE_MENU_ITEMS,
+		RESET_MENU_ITEMS,
+		SHOW_MENU_BAR_ITEM,
+		ENC_INPUT_ITEMS,
+		NULL);
+	menutree_apply_shortcuts(menu_tree, shortcuts);
+}
+
 static void menutree_destroy(MenuTree * tree)
 {
 	if (tree->deleted_handler)
@@ -524,12 +565,36 @@ static void menutree_destroy_widget_handler(GtkObject * obj, MenuTree * tree)
 	menutree_destroy(tree);
 }
 
+static MenuTree *menutree_new_common(Options *shortcuts,
+        GtkAccelGroup *accel_group,
+        GType menu_type,
+        void (*builder)(MenuTree *menu_tree, Options *shortcuts,
+                GType menu_type),
+        gboolean disable_shortcuts,
+        gboolean disable_tab_shortcuts, gpointer user_data)
+{
+	MenuTree *tree = g_new0(MenuTree, 1);
+
+	tree->user_data = user_data;
+	tree->accel_group = accel_group;
+	tree->disable_shortcuts = disable_shortcuts;
+	tree->disable_tab_shortcuts = disable_tab_shortcuts;
+
+	builder(tree, shortcuts, menu_type);
+
+	g_signal_connect(GTK_OBJECT(tree->top_level),
+		"destroy", G_CALLBACK(menutree_destroy_widget_handler), tree);
+
+	gtk_widget_show_all(tree->top_level);
+	return tree;
+}
+
 MenuTree *menutree_new(Options *shortcuts, GtkAccelGroup *accel_group,
 	GType menu_type, gboolean disable_shortcuts,
 	gboolean disable_tab_shortcuts, gpointer user_data)
 {
-	MenuTree *tree = g_new0(MenuTree, 1);
-
+	MenuTree *tree = NULL;
+	
 	if (!filled_labels)
 	{
 		int n;
@@ -537,23 +602,20 @@ MenuTree *menutree_new(Options *shortcuts, GtkAccelGroup *accel_group,
 		for (n = 0; n < MENUTREE_NUM_IDS; ++n)
 			menutree_labels[n] = NULL;
 	}
-	tree->user_data = user_data;
-	tree->accel_group = accel_group;
-	tree->disable_shortcuts = disable_shortcuts;
-	tree->disable_tab_shortcuts = disable_tab_shortcuts;
-
-	menutree_build(tree, shortcuts, menu_type);
-
-	g_signal_connect(GTK_OBJECT(tree->top_level),
-		"destroy", G_CALLBACK(menutree_destroy_widget_handler), tree);
-
-	gtk_widget_show_all(tree->top_level);
-
+    tree = menutree_new_common(shortcuts, accel_group, menu_type,
+        menutree_build, disable_shortcuts, disable_tab_shortcuts, user_data);
 	filled_labels = TRUE;
-
 	return tree;
 }
 
+MenuTree *menutree_new_short_popup(Options *shortcuts,
+        GtkAccelGroup *accel_group, gboolean disable_shortcuts,
+        gpointer user_data)
+{
+    return menutree_new_common(shortcuts, accel_group, GTK_TYPE_MENU,
+        menutree_build_short_popup, disable_shortcuts, FALSE, user_data);
+}
+	
 void menutree_delete(MenuTree * tree)
 {
 	g_return_if_fail(tree != NULL);
@@ -643,26 +705,52 @@ GtkWidget *menutree_add_tab_at_position(MenuTree * tree, const char *title,
 	return menu_item;
 }
 
+static void menu_item_change_label(GtkMenuItem *item, const char *label)
+{
+    GtkWidget *inner_item = gtk_bin_get_child(GTK_BIN(item));
+    
+    if (GTK_IS_LABEL(inner_item))
+    {
+        gtk_label_set_text_with_mnemonic(GTK_LABEL(inner_item), label);
+    }
+    else if (GTK_IS_CONTAINER(inner_item))
+    {
+        GList *link;
+        
+        for (link = gtk_container_get_children(GTK_CONTAINER(inner_item));
+                link; link = g_list_next(link))
+        {
+            GtkWidget *child = link->data;
+            
+            if (GTK_IS_LABEL(child))
+            {
+                gtk_label_set_text_with_mnemonic(GTK_LABEL(child), label);
+                return;
+            }
+        }
+    }
+}
+
+inline static void menutree_change_label(MenuTree *tree, MenuTreeID id,
+        const char *label)
+{
+    menu_item_change_label(GTK_MENU_ITEM(menutree_get_widget_for_id(tree, id)),
+            label);
+}
+
 GtkWidget *menutree_change_tab_title(MenuTree * tree,
 	GtkWidget * menu_item, const char *title)
 {
-	GtkMenuShell *submenu = menutree_get_submenu_at_id(tree, MENUTREE_TABS);
-	int position = g_list_index(tree->tabs, menu_item);
-	gboolean selected = gtk_check_menu_item_get_active(
-			GTK_CHECK_MENU_ITEM(menu_item));
+    menu_item_change_label(GTK_MENU_ITEM(menu_item), title);
+    return menu_item;
+}
 
-	menutree_remove_tab_without_fixing_accels(tree, menu_item);
-	menu_item = menutree_tab_menu_item_new(submenu, title);
-	gtk_menu_shell_insert(submenu, menu_item,
-			position + MENUTREE_TABS_FIRST_DYNAMIC);
-	tree->tabs = g_list_insert(tree->tabs, menu_item, position);
-	if (++tree->ntabs > 1)
-		menutree_set_accel_path_for_tab(tree, position);
-	if (selected)
-	{
-		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menu_item), TRUE);
-	}
-	return menu_item;
+void menutree_change_move_tab_labels(MenuTree *tree)
+{
+    menutree_change_label(tree, MENUTREE_TABS_MOVE_TAB_LEFT,
+            _("Move Tab _Up"));
+    menutree_change_label(tree, MENUTREE_TABS_MOVE_TAB_RIGHT,
+            _("Move Tab _Down"));
 }
 
 static void menutree_set_toggle(MenuTree *tree, MenuTreeID id, gboolean active)
@@ -694,7 +782,8 @@ gulong menutree_named_signal_connect_data(MenuTree * tree, MenuTreeID id,
 
 	g_return_val_if_fail(tree, 0);
 	menu_item = menutree_get_widget_for_id(tree, id);
-	g_return_val_if_fail(menu_item, 0);
+	if (!menu_item)
+	    return 0;
 	return g_signal_connect_data(menu_item, signame, handler, user_data,
 		NULL, flags);
 }
