@@ -17,8 +17,8 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-
 #include "dlg.h"
+#include "display.h"
 #include "globalopts.h"
 #include "menutree.h"
 #include "multitab.h"
@@ -46,11 +46,14 @@ struct MultiTab {
     double scroll_step;
     gboolean attention;
     guint attention_tag;
+    GtkWidget *close_button;
+    GtkWidget *label_box;
+    void (*label_packer)(GtkBox *, GtkWidget *, gboolean, gboolean, guint);
 };
 
 struct MultiWin {
     GtkWidget *gtkwin;        /* Top-level window */
-    GtkWidget *vbox;        /* Container for menu bar, tabs and vte widget */
+    GtkWidget *vbox;          /* Container for menu bar, tabs and vte widget */
     GtkWidget *notebook;
     MenuTree *menu_bar;
     MenuTree *popup_menu;
@@ -59,6 +62,7 @@ struct MultiWin {
     GList *tabs;
     MultiTab *current_tab;
     GtkPositionType tab_pos;
+    gboolean always_show_tabs;
     gpointer user_data_template;
     gulong destroy_handler;
     gboolean ignore_tab_selections;
@@ -73,7 +77,7 @@ struct MultiWin {
     int child_width, child_height; /* Need to store size of child widget before
                                       removing or adding a tab so we can
                                       correct size afterwards */
-    int ignore_size_allocate;        /* 0 = Don't ignore
+    int ignore_size_allocate;       /* 0 = Don't ignore
                                        1 = Ignore
                                        2 = Special case after tab label has
                                            changed */
@@ -95,6 +99,7 @@ struct MultiWin {
 #if HAVE_COMPOSITE
     gboolean composite;
 #endif
+    char *display_name;
 };
 
 static double multi_win_zoom_factors[] = {
@@ -128,6 +133,7 @@ static MultiWinZoomHandler multi_win_zoom_handler;
 static MultiWinGetDisableMenuShortcuts multi_win_get_disable_menu_shortcuts;
 static MultiWinInitialTabs multi_win_initial_tabs;
 static MultiWinDeleteHandler multi_win_delete_handler;
+static MultiTabGetShowCloseButton multi_tab_get_show_close_button;
 
 static gboolean multi_win_notify_tab_removed(MultiWin *, MultiTab *);
 
@@ -149,15 +155,17 @@ inline static MultiTab *multi_tab_from_widget(GtkWidget *widget)
     return g_object_get_data(G_OBJECT(widget), "roxterm_tab");
 }
 
+
 #define MULTI_WIN_NOTEBOOK_GROUP 8
 
 void multi_tab_popup_menu(MultiTab * tab, guint button, guint32 event_time)
 {
-    gtk_menu_popup(GTK_MENU(menutree_get_top_level_widget
+    GtkMenu *menu = GTK_MENU(menutree_get_top_level_widget
             (tab->parent->show_menu_bar ? tab->parent->short_popup :
-                    tab->parent->popup_menu)),
-            NULL, NULL, NULL, NULL,
-            button, event_time);
+            tab->parent->popup_menu));
+    
+    gtk_menu_set_screen(menu, gtk_widget_get_screen(tab->widget));
+    gtk_menu_popup(menu, NULL, NULL, NULL, NULL, button, event_time);
 }
 
 
@@ -168,7 +176,8 @@ void multi_tab_init(MultiTabFiller filler, MultiTabDestructor destructor,
     MultiWinZoomHandler zoom_handler,
     MultiWinGetDisableMenuShortcuts get_disable_menu_shortcuts,
     MultiWinInitialTabs initial_tabs,
-    MultiWinDeleteHandler delete_handler)
+    MultiWinDeleteHandler delete_handler,
+    MultiTabGetShowCloseButton get_show_close_button)
 {
     multi_tab_filler = filler;
     multi_tab_destructor = destructor;
@@ -180,6 +189,7 @@ void multi_tab_init(MultiTabFiller filler, MultiTabDestructor destructor,
     multi_win_get_disable_menu_shortcuts = get_disable_menu_shortcuts;
     multi_win_initial_tabs = initial_tabs;
     multi_win_delete_handler = delete_handler;
+    multi_tab_get_show_close_button = get_show_close_button;
 }
 
 MultiTab *multi_tab_new(MultiWin * parent, gpointer user_data_template)
@@ -250,6 +260,11 @@ GtkWidget *multi_tab_get_widget(MultiTab * tab)
     return tab->widget;
 }
 
+const char *multi_tab_get_display_name(MultiTab *tab)
+{
+    return tab->parent ? tab->parent->display_name : NULL;
+}
+
 static void multi_tab_set_name_or_title(MultiTab *tab, const char *title)
 {
     MultiWin *win = tab->parent;
@@ -475,6 +490,7 @@ void multi_tab_remove_from_parent(MultiTab *tab, gboolean notify_only)
         g_object_ref(tab->widget);
         gtk_notebook_remove_page(GTK_NOTEBOOK(win->notebook),
                 multi_tab_get_page_num(tab));
+        tab->close_button = NULL;
     }
     multi_tab_remove_menutree_items(win, tab);
     if (!multi_win_notify_tab_removed(win, tab))
@@ -947,9 +963,10 @@ static GtkNotebook *multi_win_notebook_creation_hook(GtkNotebook *source,
         MultiWin_GetTargetSize,
         &win->child_width, &win->child_height);
     multi_tab_remove_from_parent(tab, TRUE);
-    win = multi_win_new_blank(multi_win_get_shortcut_scheme(win),
+    win = multi_win_new_blank(win->display_name,
+                multi_win_get_shortcut_scheme(win),
                 ((MultiWin *) data)->zoom_index, multi_tab_get_user_data(tab),
-                win->tab_pos);
+                win->tab_pos, win->always_show_tabs);
     multi_win_add_tab(win, tab, -1, TRUE);
     gtk_widget_show_all(win->notebook);
     /* Defer showing till page_added_callback because we can't set size
@@ -970,9 +987,10 @@ static void multi_win_new_window_action(MultiWin * win)
     int num_tabs = 1;
 
     multi_win_initial_tabs(win->current_tab->user_data, &tab_pos, &num_tabs);
-    multi_win_new(win->shortcuts, win->zoom_index,
+    multi_win_new(win->display_name, win->shortcuts, win->zoom_index,
               win->current_tab ? win->current_tab->user_data :
-              win->user_data_template, num_tabs, tab_pos);
+              win->user_data_template, num_tabs, tab_pos,
+              win->always_show_tabs);
 }
 
 static void multi_win_new_tab_action(MultiWin * win)
@@ -1265,8 +1283,9 @@ void multi_win_show(MultiWin *win)
     g_object_set_data(G_OBJECT(win->gtkwin->window), "ROXTermWin", win);
 }
 
-MultiWin *multi_win_new_blank(Options *shortcuts, int zoom_index,
-        gpointer user_data_template, GtkPositionType tab_pos)
+MultiWin *multi_win_new_blank(const char *display_name, Options *shortcuts,
+        int zoom_index, gpointer user_data_template, GtkPositionType tab_pos,
+        gboolean always_show_tabs)
 {
     MultiWin *win = g_new0(MultiWin, 1);
     gboolean disable_menu_shortcuts, disable_tab_menu_shortcuts;
@@ -1286,6 +1305,7 @@ MultiWin *multi_win_new_blank(Options *shortcuts, int zoom_index,
 #endif
 
     win->tab_pos = tab_pos;
+    win->always_show_tabs = always_show_tabs;
     win->scroll_bar_pos = MultiWinScrollBar_Query;
     win->user_data_template = user_data_template;
     if (zoom_index < 0)
@@ -1294,6 +1314,15 @@ MultiWin *multi_win_new_blank(Options *shortcuts, int zoom_index,
         win->zoom_index = zoom_index;
 
     win->gtkwin = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+    if (display_name)
+    {
+        GdkScreen *scr;
+        
+        win->display_name = g_strdup(display_name);
+        scr = display_get_screen_for_name(display_name);
+        if (scr)
+            gtk_window_set_screen(GTK_WINDOW(win->gtkwin), scr);
+    }
     g_signal_connect(win->gtkwin, "realize",
             G_CALLBACK(multi_win_realize_handler), win);
     win->destroy_handler = g_signal_connect(win->gtkwin, "destroy",
@@ -1362,7 +1391,10 @@ MultiWin *multi_win_new_blank(Options *shortcuts, int zoom_index,
 
     win->notebook = gtk_notebook_new();
     gtk_notebook_set_scrollable(GTK_NOTEBOOK(win->notebook), TRUE);
-    multi_win_hide_tabs(win);
+    if (always_show_tabs)
+        multi_win_show_tabs(win);
+    else
+        multi_win_hide_tabs(win);
     gtk_box_pack_start_defaults(GTK_BOX(win->vbox), win->notebook);
     g_signal_connect(win->notebook, "switch-page",
         G_CALLBACK(multi_win_page_switched), win);
@@ -1386,12 +1418,13 @@ MultiWin *multi_win_new_blank(Options *shortcuts, int zoom_index,
     return win;
 }
 
-MultiWin *multi_win_new_full(Options *shortcuts, int zoom_index,
-                 gpointer user_data_template, const char *geom,
-                 gboolean fullscreen, int numtabs, GtkPositionType tab_pos)
+MultiWin *multi_win_new_full(const char *display_name, Options *shortcuts,
+        int zoom_index, gpointer user_data_template, const char *geom,
+        gboolean fullscreen, int numtabs, GtkPositionType tab_pos,
+        gboolean always_show_tabs)
 {
-    MultiWin *win = multi_win_new_blank(shortcuts, zoom_index,
-            user_data_template, tab_pos);
+    MultiWin *win = multi_win_new_blank(display_name, shortcuts, zoom_index,
+            user_data_template, tab_pos, always_show_tabs);
 
     win->tab_pos = tab_pos;
     while(numtabs--)
@@ -1399,9 +1432,16 @@ MultiWin *multi_win_new_full(Options *shortcuts, int zoom_index,
 
     gtk_notebook_set_current_page(GTK_NOTEBOOK(win->notebook), 0);
     if (tab_pos == -1)
-        gtk_notebook_set_show_tabs(GTK_NOTEBOOK(win->notebook), FALSE);
+    {
+        gtk_notebook_set_show_tabs(GTK_NOTEBOOK(win->notebook),
+                always_show_tabs);
+        gtk_notebook_set_show_border(GTK_NOTEBOOK(win->notebook),
+                always_show_tabs);
+    }
     else
+    {
         gtk_notebook_set_tab_pos(GTK_NOTEBOOK(win->notebook), tab_pos);
+    }
     multi_win_shade_menus_for_tabs(win);
 
     if (geom)
@@ -1459,6 +1499,10 @@ static void multi_win_destructor(MultiWin *win, gboolean destroy_widgets)
         win->gtkwin = NULL;
     }
     UNREF_LOG(options_unref(win->shortcuts));
+    if (win->display_name)
+    {
+        g_free(win->display_name);
+    }
     g_free(win);
     multi_win_all = g_list_remove(multi_win_all, win);
     if (!multi_win_all)
@@ -1493,7 +1537,7 @@ static gboolean multi_win_notify_tab_removed(MultiWin * win, MultiTab * tab)
     }
     else
     {
-        if (win->ntabs == 1)
+        if (win->ntabs == 1 && !win->always_show_tabs)
         {
             multi_win_hide_tabs(win);
         }
@@ -1508,18 +1552,81 @@ static gboolean multi_win_notify_tab_removed(MultiWin * win, MultiTab * tab)
     return FALSE;
 }
 
+void multi_tab_add_close_button(MultiTab *tab)
+{
+    GtkWidget *image;
+    GtkRcStyle *rcstyle;
+    GtkSettings *settings;
+    int w, h;
+    MultiWin *win;
+
+    if (tab->close_button)
+        return;
+    
+    win = tab->parent;
+    if (win)
+        win->ignore_size_allocate = 1;
+    tab->close_button = gtk_button_new();
+    image = gtk_image_new_from_stock(GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU);
+    rcstyle = gtk_rc_style_new();
+    gtk_container_add(GTK_CONTAINER(tab->close_button), image);
+    gtk_button_set_relief(GTK_BUTTON(tab->close_button), GTK_RELIEF_NONE);
+    gtk_button_set_focus_on_click(GTK_BUTTON(tab->close_button), FALSE);
+    settings = gtk_widget_get_settings(tab->label);
+    gtk_icon_size_lookup_for_settings(settings, GTK_ICON_SIZE_MENU, &w, &h);
+    gtk_widget_set_size_request(tab->close_button, w + 2, h + 2);
+    rcstyle->xthickness = rcstyle->ythickness = 0;
+    gtk_widget_modify_style(tab->close_button, rcstyle);
+    gtk_rc_style_unref(rcstyle);
+    tab->label_packer(GTK_BOX(tab->label_box), tab->close_button,
+            FALSE, FALSE, 0);
+    g_signal_connect(tab->close_button, "clicked",
+            G_CALLBACK(multi_win_close_tab_clicked), tab);
+    gtk_widget_show_all(tab->close_button);
+    if (win)
+    {
+        if (win->current_tab)
+        {
+            multi_win_set_size(win, win->current_tab->active_widget,
+                win->child_width, win->child_height);
+        }
+        else
+        {
+            win->ignore_size_allocate = 0;
+        }
+    }
+}
+
+void multi_tab_remove_close_button(MultiTab *tab)
+{
+    if (tab->close_button)
+    {
+        MultiWin *win;
+    
+        win = tab->parent;
+        if (win)
+            win->ignore_size_allocate = 1;
+        gtk_widget_destroy(tab->close_button);
+        tab->close_button = NULL;
+        if (win)
+        {
+            if (win->current_tab)
+            {
+                multi_win_set_size(win, win->current_tab->active_widget,
+                    win->child_width, win->child_height);
+            }
+            else
+            {
+                win->ignore_size_allocate = 0;
+            }
+        }
+    }
+}
+
 /* Creates the label widget for a tab. tab->label is the GtkLabel containing
  * the text; the return value is the top-level container. */
 static GtkWidget *make_tab_label(MultiTab *tab, GtkPositionType tab_pos)
 {
-    GtkWidget *hbox;
-    void (*packer)(GtkBox *, GtkWidget *, gboolean, gboolean, guint);
-    GtkWidget *close_button = gtk_button_new();
-    GtkWidget *image = gtk_image_new_from_stock(GTK_STOCK_CLOSE,
-            GTK_ICON_SIZE_MENU);
-    GtkRcStyle *rcstyle = gtk_rc_style_new();
-    GtkSettings *settings;
-    int w, h;
     static GdkColor amber;
     static gboolean parsed_amber = FALSE;
 
@@ -1528,29 +1635,19 @@ static GtkWidget *make_tab_label(MultiTab *tab, GtkPositionType tab_pos)
     {
         case GTK_POS_LEFT:
         case GTK_POS_RIGHT:
-            hbox = gtk_vbox_new(FALSE, 4);
-            packer = gtk_box_pack_end;
+            tab->label_box = gtk_vbox_new(FALSE, 4);
+            tab->label_packer = gtk_box_pack_end;
             gtk_label_set_ellipsize(GTK_LABEL(tab->label),
                     PANGO_ELLIPSIZE_NONE);
             break;
         default:
-            hbox = gtk_hbox_new(FALSE, 4);
-            packer = gtk_box_pack_start;
+            tab->label_box = gtk_hbox_new(FALSE, 4);
+            tab->label_packer = gtk_box_pack_start;
             gtk_label_set_ellipsize(GTK_LABEL(tab->label),
                     PANGO_ELLIPSIZE_MIDDLE);
              break;
     }
     
-    gtk_container_add(GTK_CONTAINER(close_button), image);
-    gtk_button_set_relief(GTK_BUTTON(close_button), GTK_RELIEF_NONE);
-    gtk_button_set_focus_on_click(GTK_BUTTON(close_button), FALSE);
-    settings = gtk_widget_get_settings(tab->label);
-    gtk_icon_size_lookup_for_settings(settings, GTK_ICON_SIZE_MENU, &w, &h);
-    gtk_widget_set_size_request(close_button, w + 2, h + 2);
-    rcstyle->xthickness = rcstyle->ythickness = 0;
-    gtk_widget_modify_style(close_button, rcstyle);
-    gtk_rc_style_unref(rcstyle);
-    
     tab->label_bg = gtk_event_box_new();
     if (!parsed_amber)
     {
@@ -1560,21 +1657,27 @@ static GtkWidget *make_tab_label(MultiTab *tab, GtkPositionType tab_pos)
     gtk_widget_modify_bg(tab->label_bg, GTK_STATE_NORMAL, &amber);
     gtk_widget_modify_bg(tab->label_bg, GTK_STATE_ACTIVE, &amber);
     gtk_event_box_set_visible_window(GTK_EVENT_BOX(tab->label_bg), FALSE);
-    packer(GTK_BOX(hbox), tab->label_bg, TRUE, TRUE, 0);
+    tab->label_packer(GTK_BOX(tab->label_box), tab->label_bg, TRUE, TRUE, 0);
     gtk_container_add(GTK_CONTAINER(tab->label_bg), tab->label);
 #if DO_OWN_TAB_DRAGGING
     tab->tab_drag = tab_drag_new(tab->label_bg, tab);
     g_signal_connect(tab->label_bg, "button-press-event",
             G_CALLBACK(tab_clicked_handler), tab);
 #endif
-    packer(GTK_BOX(hbox), close_button, FALSE, FALSE, 0);
+    if (multi_tab_get_show_close_button(tab->user_data))
+    {
+        g_debug("Adding close button");
+        multi_tab_add_close_button(tab);
+    }
+    else
+    {
+        g_debug("Not adding close button");
+    }
     gtk_notebook_set_tab_label(GTK_NOTEBOOK(tab->parent->notebook),
-        tab->widget, hbox);
-    g_signal_connect(close_button, "clicked",
-            G_CALLBACK(multi_win_close_tab_clicked), tab);
-    gtk_widget_show_all(hbox);
+        tab->widget, tab->label_box);
+    gtk_widget_show_all(tab->label_box);
 
-    return hbox;
+    return tab->label_box;
 }
 
 static void multi_tab_add_menutree_items(MultiWin * win, MultiTab * tab,
@@ -1840,6 +1943,31 @@ GtkPositionType multi_win_get_tab_pos(MultiWin *win)
     return win->tab_pos;
 }
 
+gboolean multi_win_get_always_show_tabs(MultiWin *win)
+{
+    return win->always_show_tabs;
+}
+
+void multi_win_set_always_show_tabs(MultiWin *win, gboolean show)
+{
+    gboolean old_show = win->always_show_tabs;
+    
+    win->always_show_tabs = show;
+    if (win->ntabs == 1 && old_show != show)
+    {
+        MultiTab *tab = win->current_tab;
+        int w, h;
+        
+        g_return_if_fail(tab != NULL);
+        (*multi_win_size_func)(tab->user_data, MultiWin_GetTargetSize, &w, &h);
+        if (show)
+            multi_win_show_tabs(win);
+        else
+            multi_win_hide_tabs(win);
+        multi_win_set_size(win, tab->active_widget, w, h);
+    }
+}
+
 static gboolean check_title_template(const char *tt)
 {
     const char *c;
@@ -1927,6 +2055,11 @@ gboolean multi_win_composite(MultiWin *win)
 #endif
 }
 
+const char *multi_win_get_display_name(MultiWin *win)
+{
+    return win->display_name;
+}
+
 /* This is maintained in order of most recently focused */
 GList *multi_win_all = NULL;
 
