@@ -92,6 +92,9 @@ typedef void (*MultiWinInitialTabs)(gpointer user_data,
 /* A handler for delete-event */
 typedef void (*MultiWinDeleteHandler)(GtkWidget *, GdkEvent *, MultiWin *win);
 
+/* Return whether to show close buttons in tabs */
+typedef gboolean (*MultiTabGetShowCloseButton)(gpointer user_data);
+
 /* Call to set up function hooks. See MultiTabFiller etc above.
  * menu_signal_connector is called each time a new window is created to give
  * the client a chance to connect its signal handlers; each handler will
@@ -101,7 +104,7 @@ multi_tab_init(MultiTabFiller filler, MultiTabDestructor destructor,
     MultiWinMenuSignalConnector menu_signal_connector,
     MultiWinGeometryFunc, MultiWinSizeFunc, MultiTabToNewWindowHandler,
     MultiWinZoomHandler, MultiWinGetDisableMenuShortcuts, MultiWinInitialTabs,
-    MultiWinDeleteHandler);
+    MultiWinDeleteHandler, MultiTabGetShowCloseButton);
 
 /* Register a MultiTabSelectionHandler (see above) */
 void
@@ -114,6 +117,8 @@ void multi_tab_delete(MultiTab *);
 
 GtkWidget *multi_tab_get_widget(MultiTab *);
 
+const char *multi_tab_get_display_name(MultiTab *tab);
+
 void multi_tab_set_title(MultiTab *, const char *);
 
 void multi_tab_set_name(MultiTab *, const char *);
@@ -161,41 +166,57 @@ MultiTab *multi_tab_get_tab_under_pointer(int x, int y);
 void multi_tab_draw_attention(MultiTab *tab);
 void multi_tab_cancel_attention(MultiTab *tab);
 
+void multi_tab_add_close_button(MultiTab *tab);
+void multi_tab_remove_close_button(MultiTab *tab);
+
 /* In following functions a zoom_index of -1 means default */
 
 /* Create a new window with default settings, adding it to main list */
-MultiWin *multi_win_new_full(Options *shortcuts, int zoom_index,
-                 gpointer user_data_template, const char *geom,
-                 gboolean full_screen, int numtabs, GtkPositionType tab_pos);
+MultiWin *multi_win_new_full(const char *display_name,
+        Options *shortcuts, int zoom_index,
+        gpointer user_data_template, const char *geom,
+        gboolean full_screen, int numtabs, GtkPositionType tab_pos,
+        gboolean always_show_tabs);
 
-inline static MultiWin *multi_win_new_with_geom(Options *shortcuts,
-                        int zoom_index, gpointer user_data_template,
-                        const char *geom, int numtabs, GtkPositionType tab_pos)
+inline static MultiWin *multi_win_new_with_geom(const char *display_name,
+        Options *shortcuts,
+        int zoom_index, gpointer user_data_template,
+        const char *geom, int numtabs, GtkPositionType tab_pos,
+        gboolean always_show_tabs)
 {
-    return multi_win_new_full(shortcuts, zoom_index,
-                  user_data_template, geom, FALSE, numtabs, tab_pos);
+    return multi_win_new_full(display_name, shortcuts, zoom_index,
+                  user_data_template, geom, FALSE, numtabs, tab_pos,
+                  always_show_tabs);
 }
 
-inline static MultiWin *multi_win_new(Options *shortcuts, int zoom_index,
-      gpointer user_data_template, int numtabs, GtkPositionType tab_pos)
+inline static MultiWin *multi_win_new(const char *display_name,
+        Options *shortcuts, int zoom_index,
+        gpointer user_data_template, int numtabs, GtkPositionType tab_pos,
+        gboolean always_show_tabs)
 {
-    return multi_win_new_full(shortcuts, zoom_index,
-                  user_data_template, NULL, FALSE, numtabs, tab_pos);
+    return multi_win_new_full(display_name, shortcuts, zoom_index,
+                  user_data_template, NULL, FALSE, numtabs, tab_pos,
+                  always_show_tabs);
 }
 
-inline static MultiWin *multi_win_new_fullscreen(Options *shortcuts,
-                         int zoom_index, gpointer user_data_template,
-                         int numtabs, GtkPositionType tab_pos)
+inline static MultiWin *multi_win_new_fullscreen(const char *display_name,
+        Options *shortcuts,
+        int zoom_index, gpointer user_data_template,
+        int numtabs, GtkPositionType tab_pos,
+        gboolean always_show_tabs)
 {
-    return multi_win_new_full(shortcuts, zoom_index,
-                  user_data_template, NULL, TRUE, numtabs, tab_pos);
+    return multi_win_new_full(display_name, shortcuts, zoom_index,
+                  user_data_template, NULL, TRUE, numtabs, tab_pos,
+                  always_show_tabs);
 }
 
 /* Create a new window without a new child tab, ready to have an existing
  * tab moved to it. Window isn't shown yet because we don't know how big we
  * want it to be until it contains a tab. */
-MultiWin *multi_win_new_blank(Options *shortcuts, int zoom_index,
-        gpointer user_data_template, GtkPositionType tab_pos);
+MultiWin *multi_win_new_blank(const char *display_name,
+        Options *shortcuts, int zoom_index,
+        gpointer user_data_template, GtkPositionType tab_pos,
+        gboolean always_show_tabs);
 
 void multi_win_show(MultiWin *win);
 
@@ -306,6 +327,10 @@ void multi_win_foreach_tab(MultiWin *, MultiWinForEachTabFunc,
 
 GtkPositionType multi_win_get_tab_pos(MultiWin *win);
 
+gboolean multi_win_get_always_show_tabs(MultiWin *win);
+
+void multi_win_set_always_show_tabs(MultiWin *win, gboolean show);
+
 /* List of all known windows - treat as read-only */
 extern GList *multi_win_all;
 
@@ -319,6 +344,8 @@ void multi_win_set_title_template(MultiWin *win, const char *tt,
 /* Whether window is managed by a compositor */
 gboolean multi_win_composite(MultiWin *win);
 
+const char *multi_win_get_display_name(MultiWin *win);
+
 #endif /* MULTITAB_H */
 
 /* vi:set sw=4 ts=4 noet cindent cino= */
