@@ -18,6 +18,7 @@
 */
 
 
+#include "display.h"
 #include "dlg.h"
 #include "globalopts.h"
 #include "optsdbus.h"
@@ -48,12 +49,18 @@
 
 #ifdef ROXTERM_CAPPLET
 
-static gboolean optsdbus_read_method_arg(DBusMessage *message,
-		char RTDBUS_ARG_CONST **arg,
+static gboolean optsdbus_read_args(DBusMessage *message,
+		char RTDBUS_ARG_CONST **arg, char RTDBUS_ARG_CONST **display,
 		DBusError *perror, DBusMessage **preply_error)
 {
-	if (!dbus_message_get_args(message, perror,
-				DBUS_TYPE_STRING, arg, DBUS_TYPE_INVALID))
+    gboolean result = arg ? dbus_message_get_args(message, perror,
+				DBUS_TYPE_STRING, arg,
+				DBUS_TYPE_STRING, display,
+				DBUS_TYPE_INVALID) : 
+				dbus_message_get_args(message, perror,
+				DBUS_TYPE_STRING, display,
+				DBUS_TYPE_INVALID);
+	if (!result)
 	{
 		rtdbus_whinge(perror, _("Unable to get argument from D-BUS message"));
 		if (!dbus_message_get_no_reply(message))
@@ -78,6 +85,9 @@ static DBusHandlerResult optsdbus_method_handler(DBusConnection *connection,
 		OpenConfiglet
 	} action = Unknown;
 	RTDBUS_ARG_CONST char *profile_name = NULL;
+	RTDBUS_ARG_CONST char *display_name = NULL;
+	char RTDBUS_ARG_CONST **parg = NULL;
+	GdkScreen *scrn = NULL;
 
 	dbus_error_init(&derror);
 
@@ -95,27 +105,31 @@ static DBusHandlerResult optsdbus_method_handler(DBusConnection *connection,
 	{
 		action = OpenConfiglet;
 	}
+	if (action == EditProfile || action == EditColourScheme)
+	    parg = &profile_name;
 	switch (action)
 	{
 		case EditProfile:
 		case EditColourScheme:
-			if (!optsdbus_read_method_arg(message,
-						&profile_name, &derror, &reply))
+		case OpenConfiglet:
+			if (!optsdbus_read_args(message, parg, &display_name,
+			        &derror, &reply))
 			{
 				break;
 			}
+			if (display_name && display_name[0])
+			    scrn = display_get_screen_for_name(display_name);
 			/* Fall-through */
-		case OpenConfiglet:
 			switch (action)
 			{
 				case EditProfile:
-					profilegui_open(profile_name);
+					profilegui_open(profile_name, scrn);
 					break;
 				case EditColourScheme:
-					colourgui_open(profile_name);
+					colourgui_open(profile_name, scrn);
 					break;
 				case OpenConfiglet:
-					configlet_open();
+					configlet_open(scrn);
 					break;
 				default:
 					break;
@@ -125,6 +139,8 @@ static DBusHandlerResult optsdbus_method_handler(DBusConnection *connection,
 #if ROXTERM_DBUS_OLD_ARGS_SEMANTICS
 			if (profile_name)
 				dbus_free(profile_name);
+			if (display_name)
+				dbus_free(display_name);
 #endif
 			break;
 		default:
@@ -254,11 +270,18 @@ gboolean optsdbus_send_stuff_changed_signal(const char *what_happened,
 	return FALSE;
 }
 
-gboolean optsdbus_send_edit_opts_message(const char *method, const char *arg)
+gboolean optsdbus_send_edit_opts_message(const char *method, const char *arg,
+        const char *display_name)
 {
-	DBusMessage * message = rtdbus_method_new(OPTSDBUS_NAME,
+	DBusMessage * message;
+	
+    if (!display_name)
+        display_name = "";
+	message = rtdbus_method_new(OPTSDBUS_NAME,
 			OPTSDBUS_OBJECT_PATH, OPTSDBUS_INTERFACE, method,
-			DBUS_TYPE_STRING, RTDBUS_ARG(arg), DBUS_TYPE_INVALID);
+			DBUS_TYPE_STRING, RTDBUS_ARG(arg),
+			DBUS_TYPE_STRING, RTDBUS_ARG(display_name),
+			DBUS_TYPE_INVALID);
 
 	if (!message)
 		return FALSE;
@@ -505,7 +528,8 @@ void optsdbus_listen_for_set_shortcut_scheme_signals(
     optsdbus_set_shortcut_scheme_handler = handler;
 }
 
-gboolean optsdbus_send_edit_opts_message(const char *method, const char *arg)
+gboolean optsdbus_send_edit_opts_message(const char *method, const char *arg,
+        const char *display_name)
 {
 	GError *error = NULL;
 	char *appdir = global_options_appdir && global_options_appdir[0] ?
@@ -513,18 +537,18 @@ gboolean optsdbus_send_edit_opts_message(const char *method, const char *arg)
 	char *bindir = global_options_bindir ?
 		g_strdup_printf("%s%c", global_options_bindir, G_DIR_SEPARATOR) :
 		NULL;
-	char *command = arg
-		? g_strdup_printf("%sroxterm-config %s '--%s=%s'",
+	char *disp = display_name ? g_strconcat("--display=", display_name, NULL) :
+	        NULL;
+    char *param = arg ? g_strdup_printf("--%s=%s", method, arg) :
+            g_strdup(method);
+	char *command =  g_strdup_printf("%sroxterm-config %s %s '%s'",
 				bindir ? bindir : "", appdir ? appdir : "",
-				method, arg)
-		: g_strdup_printf("%sroxterm-config %s '--%s'",
-				bindir ? bindir : "", appdir ? appdir : "",
-				method);
+				disp ? disp : "", param);
 
-	if (bindir)
-		g_free(bindir);
-	if (appdir)
-		g_free(appdir);
+    g_free(bindir);
+    g_free(appdir);
+    g_free(disp);
+    g_free(param);
 	if (!g_spawn_command_line_async(command, &error))
 	{
 		dlg_critical(NULL, _("Error launching capplet: %s"), error->message);
