@@ -52,6 +52,9 @@ void rtdbus_whinge(DBusError *pderror, const char *s);
 /* Unrefs message after sending */
 gboolean rtdbus_send_message(DBusMessage *message);
 
+/* As above but waits for a reply and checks it for success */
+gboolean rtdbus_send_message_with_reply(DBusMessage *message);
+
 /* Returns TRUE if another instance already has the name */
 gboolean rtdbus_start_service(const char *name, const char *object_path,
 		DBusObjectPathMessageFunction, gboolean replace);
